#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Enhances a web conversion by uploading a ConversionAdjustment.

The conversion adjustment contains hashed user identifiers and an order ID.
"""


import argparse
import hashlib
import re
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException


def main(
    client,
    customer_id,
    conversion_action_id,
    order_id,
    conversion_date_time,
    user_agent,
):
    """The main method that creates all necessary entities for the example.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID string.
        conversion_action_id: The ID of the conversion action to upload to.
        order_id: The unique ID (transaction ID) of the conversion.
        conversion_date_time: The date and time of the conversion.
        user_agent: The HTTP user agent of the conversion.
    """
    # [START add_user_identifiers]
    # Extracts user email, phone, and address info from the raw data, normalizes
    # and hashes it, then wraps it in UserIdentifier objects. Creates a separate
    # UserIdentifier object for each. The data in this example is hardcoded, but
    # in your application you might read the raw data from an input file.

    # IMPORTANT: Since the identifier attribute of UserIdentifier
    # (https://developers.google.com/google-ads/api/reference/rpc/latest/UserIdentifier)
    # is a oneof
    # (https://protobuf.dev/programming-guides/proto3/#oneof-features), you must
    # set only ONE of hashed_email, hashed_phone_number, mobile_id,
    # third_party_user_id, or address_info. Setting more than one of these
    # attributes on the same UserIdentifier will clear all the other members of
    # the oneof. For example, the following code is INCORRECT and will result in
    # a UserIdentifier with ONLY a hashed_phone_number:
    #
    # incorrectly_populated_user_identifier = client.get_type("UserIdentifier")
    # incorrectly_populated_user_identifier.hashed_email = "...""
    # incorrectly_populated_user_identifier.hashed_phone_number = "...""

    raw_record = {
        # Email address that includes a period (.) before the Gmail domain.
        "email": "alex.2@example.com",
        # Address that includes all four required elements: first name, last
        # name, country code, and postal code.
        "first_name": "Alex",
        "last_name": "Quinn",
        "country_code": "US",
        "postal_code": "94045",
        # Phone number to be converted to E.164 format, with a leading '+' as
        # required.
        "phone": "+1 800 5550102",
        # This example lets you input conversion details as arguments, but in
        # reality you might store this data alongside other user data, so we
        # include it in this sample user record.
        "order_id": order_id,
        "conversion_action_id": conversion_action_id,
        "conversion_date_time": conversion_date_time,
        "currency_code": "USD",
        "user_agent": user_agent,
    }

    # Constructs the enhancement adjustment.
    conversion_adjustment = client.get_type("ConversionAdjustment")
    conversion_adjustment.adjustment_type = (
        client.enums.ConversionAdjustmentTypeEnum.ENHANCEMENT
    )

    # Creates a user identifier using the hashed email address, using the
    # normalize and hash method specifically for email addresses.
    email_identifier = client.get_type("UserIdentifier")
    # Optional: Specifies the user identifier source.
    email_identifier.user_identifier_source = (
        client.enums.UserIdentifierSourceEnum.FIRST_PARTY
    )
    # Uses the normalize and hash method specifically for email addresses.
    email_identifier.hashed_email = normalize_and_hash_email_address(
        raw_record["email"]
    )
    # Adds the email identifier to the conversion adjustment.
    conversion_adjustment.user_identifiers.append(email_identifier)

    # Checks if the record has a phone number, and if so, adds a UserIdentifier
    # for it.
    if raw_record.get("phone") is not None:
        phone_identifier = client.get_type("UserIdentifier")
        phone_identifier.hashed_phone_number = normalize_and_hash(
            raw_record["phone"]
        )
        # Adds the phone identifier to the conversion adjustment.
        conversion_adjustment.user_identifiers.append(phone_identifier)

    # Checks if the record has all the required mailing address elements, and if
    # so, adds a UserIdentifier for the mailing address.
    if raw_record.get("first_name") is not None:
        # Checks if the record contains all the other required elements of a
        # mailing address.
        required_keys = ["last_name", "country_code", "postal_code"]
        # Builds a new list of the required keys that are missing from
        # raw_record.
        missing_keys = [
            key for key in required_keys if key not in raw_record.keys()
        ]
        if len(missing_keys) > 0:
            print(
                "Skipping addition of mailing address information because the"
                f"following required keys are missing: {missing_keys}"
            )
        else:
            # Creates a user identifier using sample values for the user address,
            # hashing where required.
            address_identifier = client.get_type("UserIdentifier")
            address_info = address_identifier.address_info
            address_info.hashed_first_name = normalize_and_hash(
                raw_record["first_name"]
            )
            address_info.hashed_last_name = normalize_and_hash(
                raw_record["last_name"]
            )
            address_info.country_code = raw_record["country_code"]
            address_info.postal_code = raw_record["postal_code"]
            # Adds the address identifier to the conversion adjustment.
            conversion_adjustment.user_identifiers.append(address_identifier)
            # [END add_user_identifiers]

    # [START add_conversion_details]
    conversion_action_service = client.get_service("ConversionActionService")
    # Sets the conversion action.
    conversion_adjustment.conversion_action = (
        conversion_action_service.conversion_action_path(
            customer_id, raw_record["conversion_action_id"]
        )
    )

    # Sets the order ID. Enhancements MUST use order ID instead of GCLID
    # date/time pair.
    conversion_adjustment.order_id = order_id

    # Sets the conversion date and time if provided. Providing this value is
    # optional but recommended.
    if raw_record.get("conversion_date_time"):
        conversion_adjustment.gclid_date_time_pair.conversion_date_time = (
            raw_record["conversion_date_time"]
        )

    # Sets optional fields where a value was provided
    if raw_record.get("user_agent"):
        # Sets the user agent. This should match the user agent of the request
        # that sent the original conversion so the conversion and its
        # enhancement are either both attributed as same-device or both
        # attributed as cross-device.
        conversion_adjustment.user_agent = user_agent
        # [END add_conversion_details]

    # [START upload_enhancement]
    # Creates the conversion adjustment upload service client.
    conversion_adjustment_upload_service = client.get_service(
        "ConversionAdjustmentUploadService"
    )
    # Uploads the enhancement adjustment. Partial failure should always be set
    # to true.
    # NOTE: This request only uploads a single conversion, but if you have
    # multiple conversions to upload, it's still best to upload them in a single
    # request. See the following for per-request limits for reference:
    # https://developers.google.com/google-ads/api/docs/best-practices/quotas#conversion_upload_service
    response = conversion_adjustment_upload_service.upload_conversion_adjustments(
        customer_id=customer_id,
        conversion_adjustments=[conversion_adjustment],
        # Enables partial failure (must be true).
        partial_failure=True,
    )
    # [END upload_enhancement]

    # Prints any partial errors returned.
    # To review the overall health of your recent uploads, see:
    # https://developers.google.com/google-ads/api/docs/conversions/upload-summaries
    if response.partial_failure_error:
        print(
            "Partial error encountered: "
            f"{response.partial_failure_error.message}"
        )
    else:
        # Prints the result.
        result = response.results[0]
        print(
            f"Uploaded conversion adjustment of {result.conversion_action} for "
            f"order ID {result,order_id}."
        )


# [START normalize_and_hash]
def normalize_and_hash_email_address(email_address):
    """Returns the result of normalizing and hashing an email address.

    For this use case, Google Ads requires removal of any '.' characters
    preceding "gmail.com" or "googlemail.com"

    Args:
        email_address: An email address to normalize.

    Returns:
        A normalized (lowercase, removed whitespace) and SHA-265 hashed string.
    """
    normalized_email = email_address.strip().lower()
    email_parts = normalized_email.split("@")

    # Check that there are at least two segments
    if len(email_parts) > 1:
        # Checks whether the domain of the email address is either "gmail.com"
        # or "googlemail.com". If this regex does not match then this statement
        # will evaluate to None.
        if re.match(r"^(gmail|googlemail)\.com$", email_parts[1]):
            # Removes any '.' characters from the portion of the email address
            # before the domain if the domain is gmail.com or googlemail.com.
            email_parts[0] = email_parts[0].replace(".", "")
            normalized_email = "@".join(email_parts)

    return normalize_and_hash(normalized_email)


def normalize_and_hash(s):
    """Normalizes and hashes a string with SHA-256.

    Private customer data must be hashed during upload, as described at:
    https://support.google.com/google-ads/answer/9888656

    Args:
        s: The string to perform this operation on.

    Returns:
        A normalized (lowercase, removed whitespace) and SHA-256 hashed string.
    """
    return hashlib.sha256(s.strip().lower().encode()).hexdigest()
    # [END normalize_and_hash]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Imports offline call conversion values for calls related "
        "to your ads."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--conversion_action_id",
        type=str,
        required=True,
        help="The ID of the conversion action to upload to.",
    )
    parser.add_argument(
        "-o",
        "--order_id",
        type=str,
        required=True,
        help="the unique ID (transaction ID) of the conversion.",
    )
    parser.add_argument(
        "-d",
        "--conversion_date_time",
        type=str,
        help="The date time at which the conversion with the specified order "
        "ID occurred. Must be after the click time, and must include the time "
        "zone offset.  The format is 'yyyy-mm-dd hh:mm:ss+|-hh:mm', "
        "e.g. '2019-01-01 12:32:45-08:00'. Setting this field is optional, "
        "but recommended",
    )
    parser.add_argument(
        "-u",
        "--user_agent",
        type=str,
        help="The HTTP user agent of the conversion.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client,
            args.customer_id,
            args.conversion_action_id,
            args.order_id,
            args.conversion_date_time,
            args.user_agent,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}'' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import criteria
from google.ads.googleads.v20.common.types import dates
from google.ads.googleads.v20.common.types import keyword_plan_common
from google.ads.googleads.v20.enums.types import keyword_match_type
from google.ads.googleads.v20.enums.types import keyword_plan_keyword_annotation
from google.ads.googleads.v20.enums.types import (
    keyword_plan_network as gage_keyword_plan_network,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "GenerateKeywordIdeasRequest",
        "KeywordAndUrlSeed",
        "KeywordSeed",
        "SiteSeed",
        "UrlSeed",
        "GenerateKeywordIdeaResponse",
        "GenerateKeywordIdeaResult",
        "GenerateKeywordHistoricalMetricsRequest",
        "GenerateKeywordHistoricalMetricsResponse",
        "GenerateKeywordHistoricalMetricsResult",
        "GenerateAdGroupThemesRequest",
        "GenerateAdGroupThemesResponse",
        "AdGroupKeywordSuggestion",
        "UnusableAdGroup",
        "GenerateKeywordForecastMetricsRequest",
        "CampaignToForecast",
        "ForecastAdGroup",
        "BiddableKeyword",
        "CriterionBidModifier",
        "ManualCpcBiddingStrategy",
        "MaximizeClicksBiddingStrategy",
        "MaximizeConversionsBiddingStrategy",
        "GenerateKeywordForecastMetricsResponse",
        "KeywordForecastMetrics",
    },
)


class GenerateKeywordIdeasRequest(proto.Message):
    r"""Request message for
    [KeywordPlanIdeaService.GenerateKeywordIdeas][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateKeywordIdeas].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            The ID of the customer with the
            recommendation.
        language (str):
            The resource name of the language to target.
            Each keyword belongs to some set of languages; a
            keyword is included if language is one of its
            languages.
            If not set, all keywords will be included.

            This field is a member of `oneof`_ ``_language``.
        geo_target_constants (MutableSequence[str]):
            The resource names of the location to target.
            Maximum is 10. An empty list MAY be used to
            specify all targeting geos.
        include_adult_keywords (bool):
            If true, adult keywords will be included in
            response. The default value is false.
        page_token (str):
            Token of the page to retrieve. If not specified, the first
            page of results will be returned. To request next page of
            results use the value obtained from ``next_page_token`` in
            the previous response. The request fields must match across
            pages.
        page_size (int):
            Number of results to retrieve in a single page. A maximum of
            10,000 results may be returned, if the page_size exceeds
            this, it is ignored. If unspecified, at most 10,000 results
            will be returned. The server may decide to further limit the
            number of returned resources. If the response contains fewer
            than 10,000 results it may not be assumed as last page of
            results.
        keyword_plan_network (google.ads.googleads.v20.enums.types.KeywordPlanNetworkEnum.KeywordPlanNetwork):
            Targeting network.
            If not set, Google Search And Partners Network
            will be used.
        keyword_annotation (MutableSequence[google.ads.googleads.v20.enums.types.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation]):
            The keyword annotations to include in
            response.
        aggregate_metrics (google.ads.googleads.v20.common.types.KeywordPlanAggregateMetrics):
            The aggregate fields to include in response.
        historical_metrics_options (google.ads.googleads.v20.common.types.HistoricalMetricsOptions):
            The options for historical metrics data.
        keyword_and_url_seed (google.ads.googleads.v20.services.types.KeywordAndUrlSeed):
            A Keyword and a specific Url to generate
            ideas from for example, cars,
            www.example.com/cars.

            This field is a member of `oneof`_ ``seed``.
        keyword_seed (google.ads.googleads.v20.services.types.KeywordSeed):
            A Keyword or phrase to generate ideas from,
            for example, cars.

            This field is a member of `oneof`_ ``seed``.
        url_seed (google.ads.googleads.v20.services.types.UrlSeed):
            A specific url to generate ideas from, for
            example, www.example.com/cars.

            This field is a member of `oneof`_ ``seed``.
        site_seed (google.ads.googleads.v20.services.types.SiteSeed):
            The site to generate ideas from, for example,
            www.example.com.

            This field is a member of `oneof`_ ``seed``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    language: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    geo_target_constants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=15,
    )
    include_adult_keywords: bool = proto.Field(
        proto.BOOL,
        number=10,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=12,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=13,
    )
    keyword_plan_network: (
        gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork
    ) = proto.Field(
        proto.ENUM,
        number=9,
        enum=gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork,
    )
    keyword_annotation: MutableSequence[
        keyword_plan_keyword_annotation.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation
    ] = proto.RepeatedField(
        proto.ENUM,
        number=17,
        enum=keyword_plan_keyword_annotation.KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation,
    )
    aggregate_metrics: keyword_plan_common.KeywordPlanAggregateMetrics = (
        proto.Field(
            proto.MESSAGE,
            number=16,
            message=keyword_plan_common.KeywordPlanAggregateMetrics,
        )
    )
    historical_metrics_options: keyword_plan_common.HistoricalMetricsOptions = (
        proto.Field(
            proto.MESSAGE,
            number=18,
            message=keyword_plan_common.HistoricalMetricsOptions,
        )
    )
    keyword_and_url_seed: "KeywordAndUrlSeed" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="seed",
        message="KeywordAndUrlSeed",
    )
    keyword_seed: "KeywordSeed" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="seed",
        message="KeywordSeed",
    )
    url_seed: "UrlSeed" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="seed",
        message="UrlSeed",
    )
    site_seed: "SiteSeed" = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="seed",
        message="SiteSeed",
    )


class KeywordAndUrlSeed(proto.Message):
    r"""Keyword And Url Seed

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        url (str):
            The URL to crawl in order to generate keyword
            ideas.

            This field is a member of `oneof`_ ``_url``.
        keywords (MutableSequence[str]):
            Requires at least one keyword and no more
            than 20 keywords.
    """

    url: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    keywords: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )


class KeywordSeed(proto.Message):
    r"""Keyword Seed

    Attributes:
        keywords (MutableSequence[str]):
            Requires at least one keyword and no more
            than 20 keywords.
    """

    keywords: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )


class SiteSeed(proto.Message):
    r"""Site Seed

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        site (str):
            The domain name of the site. If the customer
            requesting the ideas doesn't own the site
            provided only public information is returned.

            This field is a member of `oneof`_ ``_site``.
    """

    site: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class UrlSeed(proto.Message):
    r"""Url Seed

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        url (str):
            The URL to crawl in order to generate keyword
            ideas.

            This field is a member of `oneof`_ ``_url``.
    """

    url: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class GenerateKeywordIdeaResponse(proto.Message):
    r"""Response message for
    [KeywordPlanIdeaService.GenerateKeywordIdeas][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateKeywordIdeas].

    Attributes:
        results (MutableSequence[google.ads.googleads.v20.services.types.GenerateKeywordIdeaResult]):
            Results of generating keyword ideas.
        aggregate_metric_results (google.ads.googleads.v20.common.types.KeywordPlanAggregateMetricResults):
            The aggregate metrics for all keyword ideas.
        next_page_token (str):
            Pagination token used to retrieve the next page of results.
            Pass the content of this string as the ``page_token``
            attribute of the next request. ``next_page_token`` is not
            returned for the last page.
        total_size (int):
            Total number of results available.
    """

    @property
    def raw_page(self):
        return self

    results: MutableSequence["GenerateKeywordIdeaResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="GenerateKeywordIdeaResult",
    )
    aggregate_metric_results: (
        keyword_plan_common.KeywordPlanAggregateMetricResults
    ) = proto.Field(
        proto.MESSAGE,
        number=4,
        message=keyword_plan_common.KeywordPlanAggregateMetricResults,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    total_size: int = proto.Field(
        proto.INT64,
        number=3,
    )


class GenerateKeywordIdeaResult(proto.Message):
    r"""The result of generating keyword ideas.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        text (str):
            Text of the keyword idea.
            As in Keyword Plan historical metrics, this text
            may not be an actual keyword, but the canonical
            form of multiple keywords. See
            KeywordPlanKeywordHistoricalMetrics message in
            KeywordPlanService.

            This field is a member of `oneof`_ ``_text``.
        keyword_idea_metrics (google.ads.googleads.v20.common.types.KeywordPlanHistoricalMetrics):
            The historical metrics for the keyword.
        keyword_annotations (google.ads.googleads.v20.common.types.KeywordAnnotations):
            The annotations for the keyword.
            The annotation data is only provided if
            requested.
        close_variants (MutableSequence[str]):
            The list of close variants from the requested
            keywords that are combined into this
            GenerateKeywordIdeaResult. See
            https://support.google.com/google-ads/answer/9342105
            for the definition of "close variants".
    """

    text: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    keyword_idea_metrics: keyword_plan_common.KeywordPlanHistoricalMetrics = (
        proto.Field(
            proto.MESSAGE,
            number=3,
            message=keyword_plan_common.KeywordPlanHistoricalMetrics,
        )
    )
    keyword_annotations: keyword_plan_common.KeywordAnnotations = proto.Field(
        proto.MESSAGE,
        number=6,
        message=keyword_plan_common.KeywordAnnotations,
    )
    close_variants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )


class GenerateKeywordHistoricalMetricsRequest(proto.Message):
    r"""Request message for
    [KeywordPlanIdeaService.GenerateKeywordHistoricalMetrics][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateKeywordHistoricalMetrics].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            The ID of the customer with the
            recommendation.
        keywords (MutableSequence[str]):
            A list of keywords to get historical metrics.
            Not all inputs will be returned as a result of
            near-exact deduplication. For example, if stats
            for "car" and "cars" are requested, only "car"
            will be returned.
            A maximum of 10,000 keywords can be used.
        language (str):
            The resource name of the language to target.
            Each keyword belongs to some set of languages; a
            keyword is included if language is one of its
            languages.
            If not set, all keywords will be included.

            This field is a member of `oneof`_ ``_language``.
        include_adult_keywords (bool):
            If true, adult keywords will be included in
            response. The default value is false.
        geo_target_constants (MutableSequence[str]):
            The resource names of the location to target.
            Maximum is 10. An empty list MAY be used to
            specify all targeting geos.
        keyword_plan_network (google.ads.googleads.v20.enums.types.KeywordPlanNetworkEnum.KeywordPlanNetwork):
            Targeting network.
            If not set, Google Search And Partners Network
            will be used.
        aggregate_metrics (google.ads.googleads.v20.common.types.KeywordPlanAggregateMetrics):
            The aggregate fields to include in response.
        historical_metrics_options (google.ads.googleads.v20.common.types.HistoricalMetricsOptions):
            The options for historical metrics data.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    keywords: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    language: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    include_adult_keywords: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    geo_target_constants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )
    keyword_plan_network: (
        gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork,
    )
    aggregate_metrics: keyword_plan_common.KeywordPlanAggregateMetrics = (
        proto.Field(
            proto.MESSAGE,
            number=8,
            message=keyword_plan_common.KeywordPlanAggregateMetrics,
        )
    )
    historical_metrics_options: keyword_plan_common.HistoricalMetricsOptions = (
        proto.Field(
            proto.MESSAGE,
            number=3,
            message=keyword_plan_common.HistoricalMetricsOptions,
        )
    )


class GenerateKeywordHistoricalMetricsResponse(proto.Message):
    r"""Response message for
    [KeywordPlanIdeaService.GenerateKeywordHistoricalMetrics][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateKeywordHistoricalMetrics].

    Attributes:
        results (MutableSequence[google.ads.googleads.v20.services.types.GenerateKeywordHistoricalMetricsResult]):
            List of keywords and their historical
            metrics.
        aggregate_metric_results (google.ads.googleads.v20.common.types.KeywordPlanAggregateMetricResults):
            The aggregate metrics for all keywords.
    """

    results: MutableSequence["GenerateKeywordHistoricalMetricsResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="GenerateKeywordHistoricalMetricsResult",
        )
    )
    aggregate_metric_results: (
        keyword_plan_common.KeywordPlanAggregateMetricResults
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        message=keyword_plan_common.KeywordPlanAggregateMetricResults,
    )


class GenerateKeywordHistoricalMetricsResult(proto.Message):
    r"""The result of generating keyword historical metrics.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        text (str):
            The text of the query associated with one or more keywords.
            Note that we de-dupe your keywords list, eliminating close
            variants before returning the keywords as text. For example,
            if your request originally contained the keywords "car" and
            "cars", the returned search query will only contain "cars".
            The list of de-duped queries will be included in
            close_variants field.

            This field is a member of `oneof`_ ``_text``.
        close_variants (MutableSequence[str]):
            The list of close variants from the requested
            keywords whose stats are combined into this
            GenerateKeywordHistoricalMetricsResult.
        keyword_metrics (google.ads.googleads.v20.common.types.KeywordPlanHistoricalMetrics):
            The historical metrics for text and its close
            variants
    """

    text: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    close_variants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    keyword_metrics: keyword_plan_common.KeywordPlanHistoricalMetrics = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message=keyword_plan_common.KeywordPlanHistoricalMetrics,
        )
    )


class GenerateAdGroupThemesRequest(proto.Message):
    r"""Request message for
    [KeywordPlanIdeaService.GenerateAdGroupThemes][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateAdGroupThemes].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        keywords (MutableSequence[str]):
            Required. A list of keywords to group into
            the provided AdGroups.
        ad_groups (MutableSequence[str]):
            Required. A list of resource names of AdGroups to group
            keywords into. Resource name format:
            ``customers/{customer_id}/adGroups/{ad_group_id}``
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    keywords: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    ad_groups: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class GenerateAdGroupThemesResponse(proto.Message):
    r"""Response message for
    [KeywordPlanIdeaService.GenerateAdGroupThemes][google.ads.googleads.v20.services.KeywordPlanIdeaService.GenerateAdGroupThemes].

    Attributes:
        ad_group_keyword_suggestions (MutableSequence[google.ads.googleads.v20.services.types.AdGroupKeywordSuggestion]):
            A list of suggested AdGroup/keyword pairings.
        unusable_ad_groups (MutableSequence[google.ads.googleads.v20.services.types.UnusableAdGroup]):
            A list of provided AdGroups that could not be
            used as suggestions.
    """

    ad_group_keyword_suggestions: MutableSequence[
        "AdGroupKeywordSuggestion"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="AdGroupKeywordSuggestion",
    )
    unusable_ad_groups: MutableSequence["UnusableAdGroup"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="UnusableAdGroup",
        )
    )


class AdGroupKeywordSuggestion(proto.Message):
    r"""The suggested text and AdGroup/Campaign pairing for a given
    keyword.

    Attributes:
        keyword_text (str):
            The original keyword text.
        suggested_keyword_text (str):
            The normalized version of keyword_text for
            BROAD/EXACT/PHRASE suggestions.
        suggested_match_type (google.ads.googleads.v20.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
            The suggested keyword match type.
        suggested_ad_group (str):
            The suggested AdGroup for the keyword. Resource name format:
            ``customers/{customer_id}/adGroups/{ad_group_id}``
        suggested_campaign (str):
            The suggested Campaign for the keyword. Resource name
            format: ``customers/{customer_id}/campaigns/{campaign_id}``
    """

    keyword_text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    suggested_keyword_text: str = proto.Field(
        proto.STRING,
        number=2,
    )
    suggested_match_type: (
        keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
    )
    suggested_ad_group: str = proto.Field(
        proto.STRING,
        number=4,
    )
    suggested_campaign: str = proto.Field(
        proto.STRING,
        number=5,
    )


class UnusableAdGroup(proto.Message):
    r"""An AdGroup/Campaign pair that could not be used as a suggestion for
    keywords.

    AdGroups may not be usable if the AdGroup

    -  belongs to a Campaign that is not ENABLED or PAUSED
    -  is itself not ENABLED

    Attributes:
        ad_group (str):
            The AdGroup resource name. Resource name format:
            ``customers/{customer_id}/adGroups/{ad_group_id}``
        campaign (str):
            The Campaign resource name. Resource name format:
            ``customers/{customer_id}/campaigns/{campaign_id}``
    """

    ad_group: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GenerateKeywordForecastMetricsRequest(proto.Message):
    r"""Request message for
    [KeywordPlanIdeaService.GenerateKeywordForecastMetrics].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            The ID of the customer.
        currency_code (str):
            The currency used for exchange rate
            conversion. By default, the account currency of
            the customer is used. Set this field only if the
            currency is different from the account currency.
            The list of valid currency codes can be found at
            https://developers.google.com/google-ads/api/data/codes-formats#currency-codes.

            This field is a member of `oneof`_ ``_currency_code``.
        forecast_period (google.ads.googleads.v20.common.types.DateRange):
            The date range for the forecast. The start
            date must be in the future and end date must be
            within 1 year from today. The reference timezone
            used is the one of the Google Ads account
            belonging to the customer. If not set, a default
            date range from next Sunday to the following
            Saturday will be used.
        campaign (google.ads.googleads.v20.services.types.CampaignToForecast):
            Required. The campaign used in the forecast.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    forecast_period: dates.DateRange = proto.Field(
        proto.MESSAGE,
        number=3,
        message=dates.DateRange,
    )
    campaign: "CampaignToForecast" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="CampaignToForecast",
    )


class CampaignToForecast(proto.Message):
    r"""A campaign to do a keyword campaign forecast.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        language_constants (MutableSequence[str]):
            The list of resource names of languages to be targeted. The
            resource name is of the format
            "languageConstants/{criterion_id}". See
            https://developers.google.com/google-ads/api/data/codes-formats#languages
            for the list of language criterion codes.
        geo_modifiers (MutableSequence[google.ads.googleads.v20.services.types.CriterionBidModifier]):
            Locations to be targeted. Locations must be
            unique.
        keyword_plan_network (google.ads.googleads.v20.enums.types.KeywordPlanNetworkEnum.KeywordPlanNetwork):
            Required. The network used for targeting.
        negative_keywords (MutableSequence[google.ads.googleads.v20.common.types.KeywordInfo]):
            The list of negative keywords to be used in
            the campaign when doing the forecast.
        bidding_strategy (google.ads.googleads.v20.services.types.CampaignToForecast.CampaignBiddingStrategy):
            Required. The bidding strategy for the
            campaign.
        conversion_rate (float):
            The expected conversion rate (number of
            conversions divided by number of total clicks)
            as defined by the user. This value is expressed
            as a decimal value, so an expected conversion
            rate of 2% should be entered as 0.02. If left
            empty, an estimated conversion rate will be
            used.

            This field is a member of `oneof`_ ``_conversion_rate``.
        ad_groups (MutableSequence[google.ads.googleads.v20.services.types.ForecastAdGroup]):
            The ad groups in the new campaign to
            forecast.
    """

    class CampaignBiddingStrategy(proto.Message):
        r"""Supported bidding strategies for new campaign forecasts.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            manual_cpc_bidding_strategy (google.ads.googleads.v20.services.types.ManualCpcBiddingStrategy):
                Use manual CPC bidding strategy for
                forecasting.

                This field is a member of `oneof`_ ``bidding_strategy``.
            maximize_clicks_bidding_strategy (google.ads.googleads.v20.services.types.MaximizeClicksBiddingStrategy):
                Use maximize clicks bidding strategy for
                forecasting.

                This field is a member of `oneof`_ ``bidding_strategy``.
            maximize_conversions_bidding_strategy (google.ads.googleads.v20.services.types.MaximizeConversionsBiddingStrategy):
                Use maximize conversions bidding strategy for
                forecasting.

                This field is a member of `oneof`_ ``bidding_strategy``.
        """

        manual_cpc_bidding_strategy: "ManualCpcBiddingStrategy" = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="bidding_strategy",
            message="ManualCpcBiddingStrategy",
        )
        maximize_clicks_bidding_strategy: "MaximizeClicksBiddingStrategy" = (
            proto.Field(
                proto.MESSAGE,
                number=2,
                oneof="bidding_strategy",
                message="MaximizeClicksBiddingStrategy",
            )
        )
        maximize_conversions_bidding_strategy: (
            "MaximizeConversionsBiddingStrategy"
        ) = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="bidding_strategy",
            message="MaximizeConversionsBiddingStrategy",
        )

    language_constants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )
    geo_modifiers: MutableSequence["CriterionBidModifier"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CriterionBidModifier",
        )
    )
    keyword_plan_network: (
        gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork,
    )
    negative_keywords: MutableSequence[criteria.KeywordInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=4,
            message=criteria.KeywordInfo,
        )
    )
    bidding_strategy: CampaignBiddingStrategy = proto.Field(
        proto.MESSAGE,
        number=5,
        message=CampaignBiddingStrategy,
    )
    conversion_rate: float = proto.Field(
        proto.DOUBLE,
        number=6,
        optional=True,
    )
    ad_groups: MutableSequence["ForecastAdGroup"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="ForecastAdGroup",
    )


class ForecastAdGroup(proto.Message):
    r"""An ad group that is part of a campaign to be forecasted.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        max_cpc_bid_micros (int):
            The max cpc to use for the ad group when
            generating forecasted traffic. This value will
            override the max cpc value set in the bidding
            strategy. Only specify this field for bidding
            strategies that max cpc values.

            This field is a member of `oneof`_ ``_max_cpc_bid_micros``.
        biddable_keywords (MutableSequence[google.ads.googleads.v20.services.types.BiddableKeyword]):
            Required. The list of biddable keywords to be
            used in the ad group when doing the forecast.
            Requires at least one keyword.
        negative_keywords (MutableSequence[google.ads.googleads.v20.common.types.KeywordInfo]):
            The details of the keyword. You should
            specify both the keyword text and match type.
    """

    max_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    biddable_keywords: MutableSequence["BiddableKeyword"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="BiddableKeyword",
    )
    negative_keywords: MutableSequence[criteria.KeywordInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message=criteria.KeywordInfo,
        )
    )


class BiddableKeyword(proto.Message):
    r"""A biddable keyword part of an ad group.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        keyword (google.ads.googleads.v20.common.types.KeywordInfo):
            Required. Keyword. Must have text and match
            type.
        max_cpc_bid_micros (int):
            A max cpc bid in micros that overrides the ad
            group level max cpc bid in forecast simulation.
            This value will override the max cpc value set
            in the bidding strategy and ad group. Only
            specify this field for bidding strategies that
            support max cpc values.

            This field is a member of `oneof`_ ``_max_cpc_bid_micros``.
    """

    keyword: criteria.KeywordInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=criteria.KeywordInfo,
    )
    max_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class CriterionBidModifier(proto.Message):
    r"""Location Criterion bid modifier.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        geo_target_constant (str):
            The resource name of the geo location to target. The
            resource name is of the format
            "geoTargetConstants/{criterion_id}".
        bid_modifier (float):
            The associated multiplier for the criterion_id. If set, this
            value cannot be 0.

            This field is a member of `oneof`_ ``_bid_modifier``.
    """

    geo_target_constant: str = proto.Field(
        proto.STRING,
        number=1,
    )
    bid_modifier: float = proto.Field(
        proto.DOUBLE,
        number=2,
        optional=True,
    )


class ManualCpcBiddingStrategy(proto.Message):
    r"""Manual CPC Bidding Strategy.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        daily_budget_micros (int):
            Campaign level budget in micros. If set, a
            minimum value is enforced for the local currency
            used in the campaign. An error will occur
            showing the minimum value if this field is set
            too low.

            This field is a member of `oneof`_ ``_daily_budget_micros``.
        max_cpc_bid_micros (int):
            Required. A bid in micros to be applied to ad
            groups within the campaign for a manual CPC
            bidding strategy.
    """

    daily_budget_micros: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    max_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=2,
    )


class MaximizeClicksBiddingStrategy(proto.Message):
    r"""Maximize Clicks Bidding Strategy.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        daily_target_spend_micros (int):
            Required. The daily target spend in micros to
            be used for estimation. A minimum value is
            enforced for the local currency used in the
            campaign. An error will occur showing the
            minimum value if this field is set too low.
        max_cpc_bid_ceiling_micros (int):
            Ceiling on max CPC bids in micros.

            This field is a member of `oneof`_ ``_max_cpc_bid_ceiling_micros``.
    """

    daily_target_spend_micros: int = proto.Field(
        proto.INT64,
        number=1,
    )
    max_cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class MaximizeConversionsBiddingStrategy(proto.Message):
    r"""Maximize Conversions Bidding Strategy.

    Attributes:
        daily_target_spend_micros (int):
            Required. The daily target spend in micros to
            be used for estimation. This value must be
            greater than zero.
    """

    daily_target_spend_micros: int = proto.Field(
        proto.INT64,
        number=1,
    )


class GenerateKeywordForecastMetricsResponse(proto.Message):
    r"""Response message for
    [KeywordPlanIdeaService.GenerateKeywordForecastMetrics].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        campaign_forecast_metrics (google.ads.googleads.v20.services.types.KeywordForecastMetrics):
            Results of the campaign forecast.

            This field is a member of `oneof`_ ``_campaign_forecast_metrics``.
    """

    campaign_forecast_metrics: "KeywordForecastMetrics" = proto.Field(
        proto.MESSAGE,
        number=1,
        optional=True,
        message="KeywordForecastMetrics",
    )


class KeywordForecastMetrics(proto.Message):
    r"""The forecast metrics for the planless keyword campaign.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        impressions (float):
            The total number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        click_through_rate (float):
            The average click through rate. Available
            only if impressions > 0.

            This field is a member of `oneof`_ ``_click_through_rate``.
        average_cpc_micros (int):
            The average cpc. Available only if clicks >
            0.

            This field is a member of `oneof`_ ``_average_cpc_micros``.
        clicks (float):
            The total number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            The total cost.

            This field is a member of `oneof`_ ``_cost_micros``.
        conversions (float):
            Forecasted number of conversions: clicks \* conversion_rate.

            This field is a member of `oneof`_ ``_conversions``.
        conversion_rate (float):
            Forecasted conversion rate.

            This field is a member of `oneof`_ ``_conversion_rate``.
        average_cpa_micros (int):
            Average cost per acquisition calculated as cost_micros /
            conversions.

            This field is a member of `oneof`_ ``_average_cpa_micros``.
    """

    impressions: float = proto.Field(
        proto.DOUBLE,
        number=1,
        optional=True,
    )
    click_through_rate: float = proto.Field(
        proto.DOUBLE,
        number=2,
        optional=True,
    )
    average_cpc_micros: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    clicks: float = proto.Field(
        proto.DOUBLE,
        number=4,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    conversions: float = proto.Field(
        proto.DOUBLE,
        number=6,
        optional=True,
    )
    conversion_rate: float = proto.Field(
        proto.DOUBLE,
        number=7,
        optional=True,
    )
    average_cpa_micros: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "FunctionErrorEnum",
    },
)


class FunctionErrorEnum(proto.Message):
    r"""Container for enum describing possible function errors."""

    class FunctionError(proto.Enum):
        r"""Enum describing possible function errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_FUNCTION_FORMAT (2):
                The format of the function is not recognized
                as a supported function format.
            DATA_TYPE_MISMATCH (3):
                Operand data types do not match.
            INVALID_CONJUNCTION_OPERANDS (4):
                The operands cannot be used together in a
                conjunction.
            INVALID_NUMBER_OF_OPERANDS (5):
                Invalid numer of Operands.
            INVALID_OPERAND_TYPE (6):
                Operand Type not supported.
            INVALID_OPERATOR (7):
                Operator not supported.
            INVALID_REQUEST_CONTEXT_TYPE (8):
                Request context type not supported.
            INVALID_FUNCTION_FOR_CALL_PLACEHOLDER (9):
                The matching function is not allowed for call
                placeholders
            INVALID_FUNCTION_FOR_PLACEHOLDER (10):
                The matching function is not allowed for the
                specified placeholder
            INVALID_OPERAND (11):
                Invalid operand.
            MISSING_CONSTANT_OPERAND_VALUE (12):
                Missing value for the constant operand.
            INVALID_CONSTANT_OPERAND_VALUE (13):
                The value of the constant operand is invalid.
            INVALID_NESTING (14):
                Invalid function nesting.
            MULTIPLE_FEED_IDS_NOT_SUPPORTED (15):
                The Feed ID was different from another Feed
                ID in the same function.
            INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA (16):
                The matching function is invalid for use with
                a feed with a fixed schema.
            INVALID_ATTRIBUTE_NAME (17):
                Invalid attribute name.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_FUNCTION_FORMAT = 2
        DATA_TYPE_MISMATCH = 3
        INVALID_CONJUNCTION_OPERANDS = 4
        INVALID_NUMBER_OF_OPERANDS = 5
        INVALID_OPERAND_TYPE = 6
        INVALID_OPERATOR = 7
        INVALID_REQUEST_CONTEXT_TYPE = 8
        INVALID_FUNCTION_FOR_CALL_PLACEHOLDER = 9
        INVALID_FUNCTION_FOR_PLACEHOLDER = 10
        INVALID_OPERAND = 11
        MISSING_CONSTANT_OPERAND_VALUE = 12
        INVALID_CONSTANT_OPERAND_VALUE = 13
        INVALID_NESTING = 14
        MULTIPLE_FEED_IDS_NOT_SUPPORTED = 15
        INVALID_FUNCTION_FOR_FEED_WITH_FIXED_SCHEMA = 16
        INVALID_ATTRIBUTE_NAME = 17


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example creates a Things to do campaign, ad group, and ad.

Prerequisite: You need to have an access to the Things to Do Center. The
integration instructions can be found at:
https://support.google.com/google-ads/answer/13387362
"""


import argparse
import sys

from examples.utils.example_helpers import get_printable_datetime
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
    MutateAdGroupAdsResponse,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
    MutateCampaignBudgetsResponse,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
    MutateCampaignsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    things_to_do_center_account_id: int,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        things_to_do_center_account_id: the Things to Do Center account ID.
    """
    # Creates a budget to be used by the campaign that will be created below.
    budget_resource_name: str = add_campaign_budget(client, customer_id)
    # Creates a Things to do campaign.
    campaign_resource_name: str = add_things_to_do_campaign(
        client,
        customer_id,
        budget_resource_name,
        things_to_do_center_account_id,
    )
    # Creates an ad group.
    ad_group_resource_name: str = add_ad_group(
        client, customer_id, campaign_resource_name
    )
    # Creates an ad group ad.
    add_ad_group_ad(client, customer_id, ad_group_resource_name)


def add_campaign_budget(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a new campaign budget in the specified customer account.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        The resource name of the newly created budget.
    """
    # Creates a campaign budget operation.
    operation: CampaignBudgetOperation = client.get_type(
        "CampaignBudgetOperation"
    )
    # Creates a campaign budget.
    campaign_budget: CampaignBudget = operation.create
    campaign_budget.name = (
        f"Interplanetary Cruise Budget #{get_printable_datetime()}"
    )
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    # Sets the amount of budget.
    campaign_budget.amount_micros = 50000000
    # Makes the budget explicitly shared. This cannot be set to false for a
    # Things to do campaign.
    campaign_budget.explicitly_shared = True

    # Issues a mutate request.
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )
    response: MutateCampaignBudgetsResponse = (
        campaign_budget_service.mutate_campaign_budgets(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name
    print(f"Added a budget with resource name: '{resource_name}'.")
    return resource_name


# [START add_things_to_do_ad]
def add_things_to_do_campaign(
    client: GoogleAdsClient,
    customer_id: str,
    budget_resource_name: str,
    things_to_do_center_account_id: int,
) -> str:
    """Creates a new Things to do campaign in the specified customer account.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        budget_resource_name: the resource name of a budget for a new campaign.
        things_to_do_center_account_id: the Things to Do Center account ID.

    Returns:
        The resource name of the newly created campaign.
    """
    # [START add_things_to_do_ad_1]
    # Creates a campaign operation.
    operation: CampaignOperation = client.get_type("CampaignOperation")
    # Creates a campaign.
    campaign: Campaign = operation.create
    campaign.name = (
        f"Interplanetary Cruise Campaign #{get_printable_datetime()}"
    )
    # Configures settings related to Things to do campaigns including
    # advertising channel type, advertising channel sub type and travel
    # campaign settings.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.TRAVEL
    )
    campaign.advertising_channel_sub_type = (
        client.enums.AdvertisingChannelSubTypeEnum.TRAVEL_ACTIVITIES
    )
    campaign.travel_campaign_settings.travel_account_id = (
        things_to_do_center_account_id
    )
    # Recommendation: Set the campaign to PAUSED when creating it to prevent
    # the ads from immediately serving. Set to ENABLED once you've added
    # targeting and the ads are ready to serve.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED
    # Sets the bidding strategy to MaximizeConversionValue. Only this type can
    # be used for Things to do campaigns.
    campaign.maximize_conversion_value = client.get_type(
        "MaximizeConversionValue"
    )
    # Sets the budget.
    campaign.campaign_budget = budget_resource_name
    # Configures the campaign network options. Only Google Search is allowed for
    # Things to do campaigns.
    campaign.network_settings.target_google_search = True
    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )
    # [END add_things_to_do_ad_1]

    # Issues a mutate request to add campaigns.
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    response: MutateCampaignsResponse = campaign_service.mutate_campaigns(
        customer_id=customer_id, operations=[operation]
    )

    resource_name: str = response.results[0].resource_name
    print(
        f"Added a Things to do campaign with resource name: '{resource_name}'."
    )
    return resource_name
    # [END add_things_to_do_ad]


# [START add_things_to_do_ad_2]
def add_ad_group(
    client: GoogleAdsClient, customer_id: str, campaign_resource_name: str
) -> str:
    """Creates a new ad group in the specified Things to do campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        campaign_resource_name: the resource name of campaign that a new ad
            group will belong to.

    Returns:
        The resource name of the newly created ad group.
    """
    # Creates an ad group operation.
    operation: AdGroupOperation = client.get_type("AdGroupOperation")
    # Creates an ad group.
    ad_group: AdGroup = operation.create
    ad_group.name = f"Earth to Mars cruise #{get_printable_datetime()}"
    # Sets the campaign.
    ad_group.campaign = campaign_resource_name
    # Sets the ad group type to TRAVEL_ADS. This is the only value allowed
    # for this field on an ad group for a Things to do campaign.
    ad_group.type_ = client.enums.AdGroupTypeEnum.TRAVEL_ADS
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED

    # Issues a mutate request to add an ad group.
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    ad_group_response: MutateAdGroupsResponse = (
        ad_group_service.mutate_ad_groups(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = ad_group_response.results[0].resource_name
    print(f"Added an ad group with resource name: '{resource_name}'.")
    return resource_name
    # [END add_things_to_do_ad_2]


# [START add_things_to_do_ad_3]
def add_ad_group_ad(
    client: GoogleAdsClient, customer_id: str, ad_group_resource_name: str
) -> None:
    """Creates a new ad group ad in the specified ad group.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_group_resource_name: the resource name of ad group that a new ad
            group ad will belong to.
    """
    # Creates an ad group ad operation.
    operation: AdGroupAdOperation = client.get_type("AdGroupAdOperation")
    # Creates a new ad group ad and sets a travel ad info.
    ad_group_ad: AdGroupAd = operation.create
    # Sets the ad group ad to enabled. Setting this to paused will cause an error
    # for Things to do campaigns. Pausing should happen at either the ad group
    # or campaign level.
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.ENABLED
    ad_group_ad.ad.travel_ad = client.get_type("TravelAdInfo")
    # Sets the ad group.
    ad_group_ad.ad_group = ad_group_resource_name

    # Issues a mutate request to add an ad group ad.
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )
    response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name
    print(f"Added an ad group ad with resource name: '{resource_name}'.")
    # [END add_things_to_do_ad_3]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Adds a Things to do campaign, ad group, and a Things to do ad."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-t",
        "--things_to_do_center_account_id",
        type=int,
        required=True,
        help=("The Things to Do Center account ID."),
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.things_to_do_center_account_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "KeywordPlanConceptGroupTypeEnum",
    },
)


class KeywordPlanConceptGroupTypeEnum(proto.Message):
    r"""Container for enumeration of keyword plan concept group
    types.

    """

    class KeywordPlanConceptGroupType(proto.Enum):
        r"""Enumerates keyword plan concept group types.

        Values:
            UNSPECIFIED (0):
                The concept group classification different
                from brand/non-brand. This is a catch all bucket
                for all classifications that are none of the
                below.
            UNKNOWN (1):
                The value is unknown in this version.
            BRAND (2):
                The concept group classification is based on
                BRAND.
            OTHER_BRANDS (3):
                The concept group classification based on
                BRAND, that didn't fit well with the BRAND
                classifications. These are generally outliers
                and can have very few keywords in this type of
                classification.
            NON_BRAND (4):
                These concept group classification is not
                based on BRAND. This is returned for generic
                keywords that don't have a brand association.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BRAND = 2
        OTHER_BRANDS = 3
        NON_BRAND = 4


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example shows how to retrieve recommendations and apply them in a batch.

Recommendations should be applied shortly after they're retrieved. Depending on
the recommendation type, a recommendation can become obsolete quickly, and
obsolete recommendations throw an error when applied. For more details, see:
https://developers.google.com/google-ads/api/docs/recommendations#take_action

As of Google Ads API v15 users can subscribe to certain recommendation types to
apply them automatically. For more details, see:
https://developers.google.com/google-ads/api/docs/recommendations#auto-apply

As of Google Ads API v16 users can proactively generate certain recommendation
types during the campaign construction process. For more details see:
https://developers.google.com/google-ads/api/docs/recommendations#recommendations-in-campaign-construction
"""


import argparse
import sys
from typing import List, Iterable

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
)
from google.ads.googleads.v22.services.types.recommendation_service import (
    ApplyRecommendationOperation,
    ApplyRecommendationResult,
)


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    detect_and_apply_recommendations(client, customer_id)


def detect_and_apply_recommendations(
    client: GoogleAdsClient, customer_id: str
) -> None:
    """Detects recommendations and applies them.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """

    # [START detect_keyword_recommendations]
    googleads_service = client.get_service("GoogleAdsService")
    query: str = """
        SELECT
          recommendation.campaign,
          recommendation.keyword_recommendation
        FROM recommendation
        WHERE
          recommendation.type = KEYWORD"""

    # Detects keyword recommendations that exist for the customer account.
    response: Iterable[GoogleAdsRow] = googleads_service.search(
        customer_id=customer_id, query=query
    )

    operations: List[ApplyRecommendationOperation] = []
    for row in response:
        recommendation = row.recommendation
        print(
            f"Keyword recommendation ('{recommendation.resource_name}') "
            f"was found for campaign '{recommendation.campaign}."
        )

        keyword = recommendation.keyword_recommendation.keyword
        print(
            f"\tKeyword = '{keyword.text}'\n" f"\tType = '{keyword.match_type}'"
        )

        # Create an ApplyRecommendationOperation that will be used to apply
        # this recommendation, and add it to the list of operations.
        operations.append(
            build_recommendation_operation(client, recommendation.resource_name)
        )
    # [END detect_keyword_recommendations]

    # If there are operations present, send a request to apply the
    # recommendations.
    if operations:
        apply_recommendations(client, customer_id, operations)


# [START build_apply_recommendation_operation]
def build_recommendation_operation(
    client: GoogleAdsClient, recommendation: str
) -> ApplyRecommendationOperation:
    """Creates a ApplyRecommendationOperation to apply the given recommendation.

    Args:
        client: an initialized GoogleAdsClient instance.
        recommendation: a resource name for the recommendation to be applied.
    """
    # If you have a recommendation ID instead of a resource name, you can create
    # a resource name like this:
    #
    # googleads_service = client.get_service("GoogleAdsService")
    # resource_name = googleads_service.recommendation_path(
    #   customer_id, recommendation.id
    # )

    operation: ApplyRecommendationOperation = client.get_type(
        "ApplyRecommendationOperation"
    )

    # Each recommendation type has optional parameters to override the
    # recommended values. Below is an example showing how to override a
    # recommended ad when a TextAdRecommendation is applied.
    #
    # operation.text_ad.ad.resource_name = "INSERT_AD_RESOURCE_NAME"
    #
    # For more details, see:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/ApplyRecommendationOperation#apply_parameters

    operation.resource_name = recommendation
    return operation
    # [END build_apply_recommendation_operation]


# [START apply_recommendation]
def apply_recommendations(
    client: GoogleAdsClient,
    customer_id: str,
    operations: List[ApplyRecommendationOperation],
) -> None:
    """Applies a batch of recommendations.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        operations: a list of ApplyRecommendationOperation messages.
    """
    # Issues a mutate request to apply the recommendations.
    recommendation_service = client.get_service("RecommendationService")
    response: ApplyRecommendationResult = (
        recommendation_service.apply_recommendation(
            customer_id=customer_id, operations=operations
        )
    )

    for result in response.results:
        print(
            "Applied a recommendation with resource name: "
            f"'{result.resource_name}'."
        )
        # [END apply_recommendation]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Retrieves keyword recommendations for specified customer and "
            "applies them."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "AccessInvitationErrorEnum",
    },
)


class AccessInvitationErrorEnum(proto.Message):
    r"""Container for enum describing possible AccessInvitation
    errors.

    """

    class AccessInvitationError(proto.Enum):
        r"""Enum describing possible AccessInvitation errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_EMAIL_ADDRESS (2):
                The email address is invalid for sending an
                invitation.
            EMAIL_ADDRESS_ALREADY_HAS_ACCESS (3):
                Email address already has access to this
                customer.
            INVALID_INVITATION_STATUS (4):
                Invalid invitation status for the operation.
            GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED (5):
                Email address cannot be like
                abc+foo@google.com.
            INVALID_INVITATION_ID (6):
                Invalid invitation ID.
            EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION (7):
                Email address already has a pending
                invitation.
            PENDING_INVITATIONS_LIMIT_EXCEEDED (8):
                Pending invitation limit exceeded for the
                customer.
            EMAIL_DOMAIN_POLICY_VIOLATED (9):
                Email address doesn't conform to the email
                domain policy. See
                https://support.google.com/google-ads/answer/2375456
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_EMAIL_ADDRESS = 2
        EMAIL_ADDRESS_ALREADY_HAS_ACCESS = 3
        INVALID_INVITATION_STATUS = 4
        GOOGLE_CONSUMER_ACCOUNT_NOT_ALLOWED = 5
        INVALID_INVITATION_ID = 6
        EMAIL_ADDRESS_ALREADY_HAS_PENDING_INVITATION = 7
        PENDING_INVITATIONS_LIMIT_EXCEEDED = 8
        EMAIL_DOMAIN_POLICY_VIOLATED = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetTypeEnum",
    },
)


class AssetTypeEnum(proto.Message):
    r"""Container for enum describing the types of asset."""

    class AssetType(proto.Enum):
        r"""Enum describing possible types of asset.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            YOUTUBE_VIDEO (2):
                YouTube video asset.
            MEDIA_BUNDLE (3):
                Media bundle asset.
            IMAGE (4):
                Image asset.
            TEXT (5):
                Text asset.
            LEAD_FORM (6):
                Lead form asset.
            BOOK_ON_GOOGLE (7):
                Book on Google asset.
            PROMOTION (8):
                Promotion asset.
            CALLOUT (9):
                Callout asset.
            STRUCTURED_SNIPPET (10):
                Structured Snippet asset.
            SITELINK (11):
                Sitelink asset.
            PAGE_FEED (12):
                Page Feed asset.
            DYNAMIC_EDUCATION (13):
                Dynamic Education asset.
            MOBILE_APP (14):
                Mobile app asset.
            HOTEL_CALLOUT (15):
                Hotel callout asset.
            CALL (16):
                Call asset.
            PRICE (17):
                Price asset.
            CALL_TO_ACTION (18):
                Call to action asset.
            DYNAMIC_REAL_ESTATE (19):
                Dynamic real estate asset.
            DYNAMIC_CUSTOM (20):
                Dynamic custom asset.
            DYNAMIC_HOTELS_AND_RENTALS (21):
                Dynamic hotels and rentals asset.
            DYNAMIC_FLIGHTS (22):
                Dynamic flights asset.
            DYNAMIC_TRAVEL (24):
                Dynamic travel asset.
            DYNAMIC_LOCAL (25):
                Dynamic local asset.
            DYNAMIC_JOBS (26):
                Dynamic jobs asset.
            LOCATION (27):
                Location asset.
            HOTEL_PROPERTY (28):
                Hotel property asset.
            DEMAND_GEN_CAROUSEL_CARD (29):
                Demand Gen Carousel Card asset.
            BUSINESS_MESSAGE (30):
                Business message asset.
            APP_DEEP_LINK (31):
                App deep link asset.
            YOUTUBE_VIDEO_LIST (32):
                YouTube video list asset.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        YOUTUBE_VIDEO = 2
        MEDIA_BUNDLE = 3
        IMAGE = 4
        TEXT = 5
        LEAD_FORM = 6
        BOOK_ON_GOOGLE = 7
        PROMOTION = 8
        CALLOUT = 9
        STRUCTURED_SNIPPET = 10
        SITELINK = 11
        PAGE_FEED = 12
        DYNAMIC_EDUCATION = 13
        MOBILE_APP = 14
        HOTEL_CALLOUT = 15
        CALL = 16
        PRICE = 17
        CALL_TO_ACTION = 18
        DYNAMIC_REAL_ESTATE = 19
        DYNAMIC_CUSTOM = 20
        DYNAMIC_HOTELS_AND_RENTALS = 21
        DYNAMIC_FLIGHTS = 22
        DYNAMIC_TRAVEL = 24
        DYNAMIC_LOCAL = 25
        DYNAMIC_JOBS = 26
        LOCATION = 27
        HOTEL_PROPERTY = 28
        DEMAND_GEN_CAROUSEL_CARD = 29
        BUSINESS_MESSAGE = 30
        APP_DEEP_LINK = 31
        YOUTUBE_VIDEO_LIST = 32


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import conversion_value_rule_status
from google.ads.googleads.v21.enums.types import value_rule_device_type
from google.ads.googleads.v21.enums.types import (
    value_rule_geo_location_match_type,
)
from google.ads.googleads.v21.enums.types import value_rule_operation


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionValueRule",
    },
)


class ConversionValueRule(proto.Message):
    r"""A conversion value rule

    Attributes:
        resource_name (str):
            Immutable. The resource name of the conversion value rule.
            Conversion value rule resource names have the form:

            ``customers/{customer_id}/conversionValueRules/{conversion_value_rule_id}``
        id (int):
            Output only. The ID of the conversion value
            rule.
        action (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleAction):
            Action applied when the rule is triggered.
        geo_location_condition (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleGeoLocationCondition):
            Condition for Geo location that must be
            satisfied for the value rule to apply.
        device_condition (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleDeviceCondition):
            Condition for device type that must be
            satisfied for the value rule to apply.
        audience_condition (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleAudienceCondition):
            Condition for audience that must be satisfied
            for the value rule to apply.
        itinerary_condition (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleItineraryCondition):
            Condition for itinerary that must be
            satisfied for the value rule to apply.
        owner_customer (str):
            Output only. The resource name of the conversion value
            rule's owner customer. When the value rule is inherited from
            a manager customer, owner_customer will be the resource name
            of the manager whereas the customer in the resource_name
            will be of the requesting serving customer. \*\* Read-only
            \*\*
        status (google.ads.googleads.v21.enums.types.ConversionValueRuleStatusEnum.ConversionValueRuleStatus):
            The status of the conversion value rule.
    """

    class ValueRuleAction(proto.Message):
        r"""Action applied when rule is applied.

        Attributes:
            operation (google.ads.googleads.v21.enums.types.ValueRuleOperationEnum.ValueRuleOperation):
                Specifies applied operation.
            value (float):
                Specifies applied value.
        """

        operation: (
            value_rule_operation.ValueRuleOperationEnum.ValueRuleOperation
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=value_rule_operation.ValueRuleOperationEnum.ValueRuleOperation,
        )
        value: float = proto.Field(
            proto.DOUBLE,
            number=2,
        )

    class ValueRuleGeoLocationCondition(proto.Message):
        r"""Condition on Geo dimension.

        Attributes:
            excluded_geo_target_constants (MutableSequence[str]):
                Geo locations that advertisers want to
                exclude.
            excluded_geo_match_type (google.ads.googleads.v21.enums.types.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType):
                Excluded Geo location match type.
            geo_target_constants (MutableSequence[str]):
                Geo locations that advertisers want to
                include.
            geo_match_type (google.ads.googleads.v21.enums.types.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType):
                Included Geo location match type.
        """

        excluded_geo_target_constants: MutableSequence[str] = (
            proto.RepeatedField(
                proto.STRING,
                number=1,
            )
        )
        excluded_geo_match_type: (
            value_rule_geo_location_match_type.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=value_rule_geo_location_match_type.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType,
        )
        geo_target_constants: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )
        geo_match_type: (
            value_rule_geo_location_match_type.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType
        ) = proto.Field(
            proto.ENUM,
            number=4,
            enum=value_rule_geo_location_match_type.ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType,
        )

    class ValueRuleDeviceCondition(proto.Message):
        r"""Condition on Device dimension.

        Attributes:
            device_types (MutableSequence[google.ads.googleads.v21.enums.types.ValueRuleDeviceTypeEnum.ValueRuleDeviceType]):
                Value for device type condition.
        """

        device_types: MutableSequence[
            value_rule_device_type.ValueRuleDeviceTypeEnum.ValueRuleDeviceType
        ] = proto.RepeatedField(
            proto.ENUM,
            number=1,
            enum=value_rule_device_type.ValueRuleDeviceTypeEnum.ValueRuleDeviceType,
        )

    class ValueRuleAudienceCondition(proto.Message):
        r"""Condition on Audience dimension.

        Attributes:
            user_lists (MutableSequence[str]):
                User Lists.
            user_interests (MutableSequence[str]):
                User Interests.
        """

        user_lists: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )
        user_interests: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class ValueRuleItineraryCondition(proto.Message):
        r"""Condition on Itinerary dimension.

        Attributes:
            advance_booking_window (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleItineraryAdvanceBookingWindow):
                Range for the number of days between the date
                of the booking and the start of the itinerary.
            travel_length (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleItineraryTravelLength):
                Range for the itinerary length in number of
                nights.
            travel_start_day (google.ads.googleads.v21.resources.types.ConversionValueRule.ValueRuleItineraryTravelStartDay):
                The days of the week on which this
                itinerary's travel can start.
        """

        advance_booking_window: (
            "ConversionValueRule.ValueRuleItineraryAdvanceBookingWindow"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ConversionValueRule.ValueRuleItineraryAdvanceBookingWindow",
        )
        travel_length: "ConversionValueRule.ValueRuleItineraryTravelLength" = (
            proto.Field(
                proto.MESSAGE,
                number=2,
                message="ConversionValueRule.ValueRuleItineraryTravelLength",
            )
        )
        travel_start_day: (
            "ConversionValueRule.ValueRuleItineraryTravelStartDay"
        ) = proto.Field(
            proto.MESSAGE,
            number=3,
            message="ConversionValueRule.ValueRuleItineraryTravelStartDay",
        )

    class ValueRuleItineraryAdvanceBookingWindow(proto.Message):
        r"""Range for the number of days between the date of the booking
        and the start of the itinerary.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            min_days (int):
                Minimum number of days between the date of
                the booking the start date.

                This field is a member of `oneof`_ ``_min_days``.
            max_days (int):
                Maximum number of days between the date of
                the booking the start date.

                This field is a member of `oneof`_ ``_max_days``.
        """

        min_days: int = proto.Field(
            proto.INT32,
            number=3,
            optional=True,
        )
        max_days: int = proto.Field(
            proto.INT32,
            number=4,
            optional=True,
        )

    class ValueRuleItineraryTravelLength(proto.Message):
        r"""Range for the itinerary length in number of nights.

        Attributes:
            min_nights (int):
                Minimum number of nights between the start
                date and the end date.
            max_nights (int):
                Maximum number of days between the start date
                and the end date.
        """

        min_nights: int = proto.Field(
            proto.INT32,
            number=1,
        )
        max_nights: int = proto.Field(
            proto.INT32,
            number=2,
        )

    class ValueRuleItineraryTravelStartDay(proto.Message):
        r"""The days of the week on which an itinerary's travel can
        start.

        Attributes:
            monday (bool):
                The travel can start on Monday.
            tuesday (bool):
                The travel can start on Tuesday.
            wednesday (bool):
                The travel can start on Wednesday.
            thursday (bool):
                The travel can start on Thursday.
            friday (bool):
                The travel can start on Friday.
            saturday (bool):
                The travel can start on Saturday.
            sunday (bool):
                The travel can start on Sunday.
        """

        monday: bool = proto.Field(
            proto.BOOL,
            number=1,
        )
        tuesday: bool = proto.Field(
            proto.BOOL,
            number=2,
        )
        wednesday: bool = proto.Field(
            proto.BOOL,
            number=3,
        )
        thursday: bool = proto.Field(
            proto.BOOL,
            number=4,
        )
        friday: bool = proto.Field(
            proto.BOOL,
            number=5,
        )
        saturday: bool = proto.Field(
            proto.BOOL,
            number=6,
        )
        sunday: bool = proto.Field(
            proto.BOOL,
            number=7,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    action: ValueRuleAction = proto.Field(
        proto.MESSAGE,
        number=3,
        message=ValueRuleAction,
    )
    geo_location_condition: ValueRuleGeoLocationCondition = proto.Field(
        proto.MESSAGE,
        number=4,
        message=ValueRuleGeoLocationCondition,
    )
    device_condition: ValueRuleDeviceCondition = proto.Field(
        proto.MESSAGE,
        number=5,
        message=ValueRuleDeviceCondition,
    )
    audience_condition: ValueRuleAudienceCondition = proto.Field(
        proto.MESSAGE,
        number=6,
        message=ValueRuleAudienceCondition,
    )
    itinerary_condition: ValueRuleItineraryCondition = proto.Field(
        proto.MESSAGE,
        number=9,
        message=ValueRuleItineraryCondition,
    )
    owner_customer: str = proto.Field(
        proto.STRING,
        number=7,
    )
    status: (
        conversion_value_rule_status.ConversionValueRuleStatusEnum.ConversionValueRuleStatus
    ) = proto.Field(
        proto.ENUM,
        number=8,
        enum=conversion_value_rule_status.ConversionValueRuleStatusEnum.ConversionValueRuleStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomerLifecycleGoalErrorEnum",
    },
)


class CustomerLifecycleGoalErrorEnum(proto.Message):
    r"""Container for enum describing possible customer lifecycle
    goal errors.

    """

    class CustomerLifecycleGoalError(proto.Enum):
        r"""Enum describing possible customer lifecycle goal errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CUSTOMER_ACQUISITION_VALUE_MISSING (2):
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.value
                must be set.
            CUSTOMER_ACQUISITION_INVALID_VALUE (3):
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.value
                must be no less than 0.01.
            CUSTOMER_ACQUISITION_INVALID_HIGH_LIFETIME_VALUE (4):
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.high_lifetime_value
                must be no less than 0.01. Also, to set this field,
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.value
                must also be present, and high_lifetime_value must be
                greater than value.
            CUSTOMER_ACQUISITION_VALUE_CANNOT_BE_CLEARED (5):
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.value
                cannot be cleared. This value would have no effect as long
                as none of your campaigns adopt the customer acquisitiong
                goal.
            CUSTOMER_ACQUISITION_HIGH_LIFETIME_VALUE_CANNOT_BE_CLEARED (6):
                CustomerLifecycleGoal.customer_acquisition_goal_value_settings.high_lifetime_value
                cannot be cleared. This value would have no effect as long
                as none of your campaigns adopt the high value optimization
                of customer acquisitiong goal.
            INVALID_EXISTING_USER_LIST (7):
                Found invalid value in
                CustomerLifecycleGoal.lifecycle_goal_customer_definition_settings.existing_user_lists.
                The userlist must be accessible, active and belong to one of
                the following types: CRM_BASED, RULE_BASED, REMARKETING.
            INVALID_HIGH_LIFETIME_VALUE_USER_LIST (8):
                Found invalid value in
                CustomerLifecycleGoal.lifecycle_goal_customer_definition_settings.high_lifetime_value_user_lists.
                The userlist must be accessible, active and belong to one of
                the following types: CRM_BASED, RULE_BASED, REMARKETING.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CUSTOMER_ACQUISITION_VALUE_MISSING = 2
        CUSTOMER_ACQUISITION_INVALID_VALUE = 3
        CUSTOMER_ACQUISITION_INVALID_HIGH_LIFETIME_VALUE = 4
        CUSTOMER_ACQUISITION_VALUE_CANNOT_BE_CLEARED = 5
        CUSTOMER_ACQUISITION_HIGH_LIFETIME_VALUE_CANNOT_BE_CLEARED = 6
        INVALID_EXISTING_USER_LIST = 7
        INVALID_HIGH_LIFETIME_VALUE_USER_LIST = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "FeedItemErrorEnum",
    },
)


class FeedItemErrorEnum(proto.Message):
    r"""Container for enum describing possible feed item errors."""

    class FeedItemError(proto.Enum):
        r"""Enum describing possible feed item errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING (2):
                Cannot convert the feed attribute value from
                string to its real type.
            CANNOT_OPERATE_ON_REMOVED_FEED_ITEM (3):
                Cannot operate on removed feed item.
            DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE (4):
                Date time zone does not match the account's
                time zone.
            KEY_ATTRIBUTES_NOT_FOUND (5):
                Feed item with the key attributes could not
                be found.
            INVALID_URL (6):
                Url feed attribute value is not valid.
            MISSING_KEY_ATTRIBUTES (7):
                Some key attributes are missing.
            KEY_ATTRIBUTES_NOT_UNIQUE (8):
                Feed item has same key attributes as another
                feed item.
            CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE (9):
                Cannot modify key attributes on an existing
                feed item.
            SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE (10):
                The feed attribute value is too large.
            LEGACY_FEED_TYPE_READ_ONLY (11):
                Feed is read only.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_CONVERT_ATTRIBUTE_VALUE_FROM_STRING = 2
        CANNOT_OPERATE_ON_REMOVED_FEED_ITEM = 3
        DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE = 4
        KEY_ATTRIBUTES_NOT_FOUND = 5
        INVALID_URL = 6
        MISSING_KEY_ATTRIBUTES = 7
        KEY_ATTRIBUTES_NOT_UNIQUE = 8
        CANNOT_MODIFY_KEY_ATTRIBUTE_VALUE = 9
        SIZE_TOO_LARGE_FOR_MULTI_VALUE_ATTRIBUTE = 10
        LEGACY_FEED_TYPE_READ_ONLY = 11


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example shows how to link a manager customer to a client customer."""

import argparse
import sys

from google.api_core import protobuf_helpers
from google.protobuf.field_mask_pb2 import FieldMask

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.customer_client_link_service.client import (
    CustomerClientLinkServiceClient,
)
from google.ads.googleads.v22.services.types.customer_client_link_service import (
    CustomerClientLinkOperation,
    MutateCustomerClientLinkResponse,
)
from google.ads.googleads.v22.resources.types.customer_client_link import (
    CustomerClientLink,
)
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchPagedResponse,
    GoogleAdsRow,
)
from google.ads.googleads.v22.services.services.customer_manager_link_service.client import (
    CustomerManagerLinkServiceClient,
)
from google.ads.googleads.v22.services.types.customer_manager_link_service import (
    CustomerManagerLinkOperation,
    MutateCustomerManagerLinkResponse,
)
from google.ads.googleads.v22.resources.types.customer_manager_link import (
    CustomerManagerLink,
)

# ManagerLinkStatusEnum is used via client.enums


# [START link_manager_to_client]
def main(
    client: GoogleAdsClient, customer_id: str, manager_customer_id: str
) -> None:
    # This example assumes that the same credentials will work for both
    # customers, but that may not be the case. If you need to use different
    # credentials for each customer, then you may either update the client
    # configuration or instantiate two clients, where at least one points to
    # a specific configuration file so that both clients don't read the same
    # file located in the $HOME dir.
    customer_client_link_service: CustomerClientLinkServiceClient = (
        client.get_service("CustomerClientLinkService")
    )

    # Extend an invitation to the client while authenticating as the manager.
    client_link_operation: CustomerClientLinkOperation = client.get_type(
        "CustomerClientLinkOperation"
    )
    client_link: CustomerClientLink = client_link_operation.create
    client_link.client_customer = customer_client_link_service.customer_path(
        customer_id
    )
    # client_link.status expects an enum value (int)
    client_link.status = client.enums.ManagerLinkStatusEnum.PENDING.value

    response: MutateCustomerClientLinkResponse = (
        customer_client_link_service.mutate_customer_client_link(
            customer_id=manager_customer_id, operation=client_link_operation
        )
    )
    resource_name: str = response.results[0].resource_name

    print(
        f'Extended an invitation from customer "{manager_customer_id}" to '
        f'customer "{customer_id}" with client link resource_name '
        f'"{resource_name}"'
    )

    # Find the manager_link_id of the link we just created, so we can construct
    # the resource name for the link from the client side. Note that since we
    # are filtering by resource_name, a unique identifier, only one
    # customer_client_link resource will be returned in the response
    query = f'''
        SELECT
            customer_client_link.manager_link_id
        FROM
            customer_client_link
        WHERE
            customer_client_link.resource_name = "{resource_name}"'''

    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    manager_link_id: int = -1  # Initialize with a default value

    try:
        search_response: SearchPagedResponse = ga_service.search(
            customer_id=manager_customer_id, query=query
        )
        # Since the googleads_service.search method returns an iterator we need
        # to initialize an iteration in order to retrieve results, even though
        # we know the query will only return a single row.
        row: GoogleAdsRow
        for row in search_response:  # Assuming direct iteration
            manager_link_id = row.customer_client_link.manager_link_id
    except GoogleAdsException as ex:
        # handle_googleads_exception(ex) # This function is not defined here
        print(f"GoogleAdsException: {ex}")  # Basic error handling
        sys.exit(1)

    customer_manager_link_service: CustomerManagerLinkServiceClient = (
        client.get_service("CustomerManagerLinkService")
    )
    manager_link_operation: CustomerManagerLinkOperation = client.get_type(
        "CustomerManagerLinkOperation"
    )
    manager_link: CustomerManagerLink = manager_link_operation.update
    manager_link.resource_name = (
        customer_manager_link_service.customer_manager_link_path(
            customer_id,
            manager_customer_id,
            manager_link_id,  # type: ignore
        )
    )

    # manager_link.status expects an enum value (int)
    manager_link.status = client.enums.ManagerLinkStatusEnum.ACTIVE.value
    # manager_link_operation.update_mask is a FieldMask
    update_mask: FieldMask = protobuf_helpers.field_mask(None, manager_link._pb)
    client.copy_from(
        manager_link_operation.update_mask,
        update_mask,
    )

    mutate_manager_link_response: MutateCustomerManagerLinkResponse = (
        customer_manager_link_service.mutate_customer_manager_link(
            customer_id=customer_id, operations=[manager_link_operation]
        )
    )
    print(
        "Client accepted invitation with resource_name: "
        f'"{mutate_manager_link_response.results[0].resource_name}"'
    )
    # [END link_manager_to_client]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Links an existing manager customer to an existing"
            "client customer"
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c", "--customer_id", type=str, required=True, help="The customer ID."
    )
    parser.add_argument(
        "-m",
        "--manager_customer_id",
        type=str,
        required=True,
        help="The manager customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")
    try:
        main(googleads_client, args.customer_id, args.manager_customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "LifecycleGoalValueSettings",
    },
)


class LifecycleGoalValueSettings(proto.Message):
    r"""Lifecycle goal value settings.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        value (float):
            Value of the lifecycle goal. For example, for
            customer acquisition goal, value is the
            incremental conversion value for new customers
            who are not of high value.

            This field is a member of `oneof`_ ``_value``.
        high_lifetime_value (float):
            High lifetime value of the lifecycle goal.
            For example, for customer acquisition goal, high
            lifetime value is the incremental conversion
            value for new customers who are of high value.
            High lifetime value should be greater than
            value, if set.

            This field is a member of `oneof`_ ``_high_lifetime_value``.
    """

    value: float = proto.Field(
        proto.DOUBLE,
        number=1,
        optional=True,
    )
    high_lifetime_value: float = proto.Field(
        proto.DOUBLE,
        number=2,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    operating_system_version_operator_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "OperatingSystemVersionConstant",
    },
)


class OperatingSystemVersionConstant(proto.Message):
    r"""A mobile operating system version or a range of versions, depending
    on ``operator_type``. List of available mobile platforms at
    https://developers.google.com/google-ads/api/reference/data/codes-formats#mobile-platforms


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the operating system
            version constant. Operating system version constant resource
            names have the form:

            ``operatingSystemVersionConstants/{criterion_id}``
        id (int):
            Output only. The ID of the operating system
            version.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            Output only. Name of the operating system.

            This field is a member of `oneof`_ ``_name``.
        os_major_version (int):
            Output only. The OS Major Version number.

            This field is a member of `oneof`_ ``_os_major_version``.
        os_minor_version (int):
            Output only. The OS Minor Version number.

            This field is a member of `oneof`_ ``_os_minor_version``.
        operator_type (google.ads.googleads.v22.enums.types.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType):
            Output only. Determines whether this constant
            represents a single version or a range of
            versions.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    os_major_version: int = proto.Field(
        proto.INT32,
        number=9,
        optional=True,
    )
    os_minor_version: int = proto.Field(
        proto.INT32,
        number=10,
        optional=True,
    )
    operator_type: (
        operating_system_version_operator_type.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=operating_system_version_operator_type.OperatingSystemVersionOperatorTypeEnum.OperatingSystemVersionOperatorType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import account_budget_proposal_status
from google.ads.googleads.v22.enums.types import account_budget_proposal_type
from google.ads.googleads.v22.enums.types import spending_limit_type
from google.ads.googleads.v22.enums.types import time_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AccountBudgetProposal",
    },
)


class AccountBudgetProposal(proto.Message):
    r"""An account-level budget proposal.

    All fields prefixed with 'proposed' may not necessarily be applied
    directly. For example, proposed spending limits may be adjusted
    before their application. This is true if the 'proposed' field has
    an 'approved' counterpart, for example, spending limits.

    Note that the proposal type (proposal_type) changes which fields are
    required and which must remain empty.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the proposal.
            AccountBudgetProposal resource names have the form:

            ``customers/{customer_id}/accountBudgetProposals/{account_budget_proposal_id}``
        id (int):
            Output only. The ID of the proposal.

            This field is a member of `oneof`_ ``_id``.
        billing_setup (str):
            Immutable. The resource name of the billing
            setup associated with this proposal.

            This field is a member of `oneof`_ ``_billing_setup``.
        account_budget (str):
            Immutable. The resource name of the
            account-level budget associated with this
            proposal.

            This field is a member of `oneof`_ ``_account_budget``.
        proposal_type (google.ads.googleads.v22.enums.types.AccountBudgetProposalTypeEnum.AccountBudgetProposalType):
            Immutable. The type of this proposal, for
            example, END to end the budget associated with
            this proposal.
        status (google.ads.googleads.v22.enums.types.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus):
            Output only. The status of this proposal.
            When a new proposal is created, the status
            defaults to PENDING.
        proposed_name (str):
            Immutable. The name to assign to the
            account-level budget.

            This field is a member of `oneof`_ ``_proposed_name``.
        approved_start_date_time (str):
            Output only. The approved start date time in
            yyyy-mm-dd hh:mm:ss format.

            This field is a member of `oneof`_ ``_approved_start_date_time``.
        proposed_purchase_order_number (str):
            Immutable. A purchase order number is a value
            that enables the user to help them reference
            this budget in their monthly invoices.

            This field is a member of `oneof`_ ``_proposed_purchase_order_number``.
        proposed_notes (str):
            Immutable. Notes associated with this budget.

            This field is a member of `oneof`_ ``_proposed_notes``.
        creation_date_time (str):
            Output only. The date time when this
            account-level budget proposal was created, which
            is not the same as its approval date time, if
            applicable.

            This field is a member of `oneof`_ ``_creation_date_time``.
        approval_date_time (str):
            Output only. The date time when this
            account-level budget was approved, if
            applicable.

            This field is a member of `oneof`_ ``_approval_date_time``.
        proposed_start_date_time (str):
            Immutable. The proposed start date time in
            yyyy-mm-dd hh:mm:ss format.

            This field is a member of `oneof`_ ``proposed_start_time``.
        proposed_start_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
            Immutable. The proposed start date time as a
            well-defined type, for example, NOW.

            This field is a member of `oneof`_ ``proposed_start_time``.
        proposed_end_date_time (str):
            Immutable. The proposed end date time in
            yyyy-mm-dd hh:mm:ss format.

            This field is a member of `oneof`_ ``proposed_end_time``.
        proposed_end_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
            Immutable. The proposed end date time as a
            well-defined type, for example, FOREVER.

            This field is a member of `oneof`_ ``proposed_end_time``.
        approved_end_date_time (str):
            Output only. The approved end date time in
            yyyy-mm-dd hh:mm:ss format.

            This field is a member of `oneof`_ ``approved_end_time``.
        approved_end_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
            Output only. The approved end date time as a
            well-defined type, for example, FOREVER.

            This field is a member of `oneof`_ ``approved_end_time``.
        proposed_spending_limit_micros (int):
            Immutable. The proposed spending limit in
            micros.  One million is equivalent to one unit.

            This field is a member of `oneof`_ ``proposed_spending_limit``.
        proposed_spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
            Immutable. The proposed spending limit as a
            well-defined type, for example, INFINITE.

            This field is a member of `oneof`_ ``proposed_spending_limit``.
        approved_spending_limit_micros (int):
            Output only. The approved spending limit in
            micros.  One million is equivalent to one unit.

            This field is a member of `oneof`_ ``approved_spending_limit``.
        approved_spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
            Output only. The approved spending limit as a
            well-defined type, for example, INFINITE.

            This field is a member of `oneof`_ ``approved_spending_limit``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=25,
        optional=True,
    )
    billing_setup: str = proto.Field(
        proto.STRING,
        number=26,
        optional=True,
    )
    account_budget: str = proto.Field(
        proto.STRING,
        number=27,
        optional=True,
    )
    proposal_type: (
        account_budget_proposal_type.AccountBudgetProposalTypeEnum.AccountBudgetProposalType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=account_budget_proposal_type.AccountBudgetProposalTypeEnum.AccountBudgetProposalType,
    )
    status: (
        account_budget_proposal_status.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus
    ) = proto.Field(
        proto.ENUM,
        number=15,
        enum=account_budget_proposal_status.AccountBudgetProposalStatusEnum.AccountBudgetProposalStatus,
    )
    proposed_name: str = proto.Field(
        proto.STRING,
        number=28,
        optional=True,
    )
    approved_start_date_time: str = proto.Field(
        proto.STRING,
        number=30,
        optional=True,
    )
    proposed_purchase_order_number: str = proto.Field(
        proto.STRING,
        number=35,
        optional=True,
    )
    proposed_notes: str = proto.Field(
        proto.STRING,
        number=36,
        optional=True,
    )
    creation_date_time: str = proto.Field(
        proto.STRING,
        number=37,
        optional=True,
    )
    approval_date_time: str = proto.Field(
        proto.STRING,
        number=38,
        optional=True,
    )
    proposed_start_date_time: str = proto.Field(
        proto.STRING,
        number=29,
        oneof="proposed_start_time",
    )
    proposed_start_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
        proto.ENUM,
        number=7,
        oneof="proposed_start_time",
        enum=time_type.TimeTypeEnum.TimeType,
    )
    proposed_end_date_time: str = proto.Field(
        proto.STRING,
        number=31,
        oneof="proposed_end_time",
    )
    proposed_end_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
        proto.ENUM,
        number=9,
        oneof="proposed_end_time",
        enum=time_type.TimeTypeEnum.TimeType,
    )
    approved_end_date_time: str = proto.Field(
        proto.STRING,
        number=32,
        oneof="approved_end_time",
    )
    approved_end_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
        proto.ENUM,
        number=22,
        oneof="approved_end_time",
        enum=time_type.TimeTypeEnum.TimeType,
    )
    proposed_spending_limit_micros: int = proto.Field(
        proto.INT64,
        number=33,
        oneof="proposed_spending_limit",
    )
    proposed_spending_limit_type: (
        spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
    ) = proto.Field(
        proto.ENUM,
        number=11,
        oneof="proposed_spending_limit",
        enum=spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
    )
    approved_spending_limit_micros: int = proto.Field(
        proto.INT64,
        number=34,
        oneof="approved_spending_limit",
    )
    approved_spending_limit_type: (
        spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
    ) = proto.Field(
        proto.ENUM,
        number=24,
        oneof="approved_spending_limit",
        enum=spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "FeedMappingErrorEnum",
    },
)


class FeedMappingErrorEnum(proto.Message):
    r"""Container for enum describing possible feed item errors."""

    class FeedMappingError(proto.Enum):
        r"""Enum describing possible feed item errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_PLACEHOLDER_FIELD (2):
                The given placeholder field does not exist.
            INVALID_CRITERION_FIELD (3):
                The given criterion field does not exist.
            INVALID_PLACEHOLDER_TYPE (4):
                The given placeholder type does not exist.
            INVALID_CRITERION_TYPE (5):
                The given criterion type does not exist.
            NO_ATTRIBUTE_FIELD_MAPPINGS (7):
                A feed mapping must contain at least one
                attribute field mapping.
            FEED_ATTRIBUTE_TYPE_MISMATCH (8):
                The type of the feed attribute referenced in
                the attribute field mapping must match the type
                of the placeholder field.
            CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED (9):
                A feed mapping for a system generated feed
                cannot be operated on.
            MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE (10):
                Only one feed mapping for a placeholder type
                is allowed per feed or customer (depending on
                the placeholder type).
            MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE (11):
                Only one feed mapping for a criterion type is
                allowed per customer.
            MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD (12):
                Only one feed attribute mapping for a
                placeholder field is allowed (depending on the
                placeholder type).
            MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD (13):
                Only one feed attribute mapping for a
                criterion field is allowed (depending on the
                criterion type).
            UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS (14):
                This feed mapping may not contain any
                explicit attribute field mappings.
            LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS (15):
                Location placeholder feed mappings can only
                be created for Places feeds.
            CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED (16):
                Mappings for typed feeds cannot be modified.
            INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED (17):
                The given placeholder type can only be mapped
                to system generated feeds.
            INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE (18):
                The given placeholder type cannot be mapped
                to a system generated feed with the given type.
            ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD (19):
                The "field" oneof was not set in an
                AttributeFieldMapping.
            LEGACY_FEED_TYPE_READ_ONLY (20):
                Feed is read only.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_PLACEHOLDER_FIELD = 2
        INVALID_CRITERION_FIELD = 3
        INVALID_PLACEHOLDER_TYPE = 4
        INVALID_CRITERION_TYPE = 5
        NO_ATTRIBUTE_FIELD_MAPPINGS = 7
        FEED_ATTRIBUTE_TYPE_MISMATCH = 8
        CANNOT_OPERATE_ON_MAPPINGS_FOR_SYSTEM_GENERATED_FEED = 9
        MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_TYPE = 10
        MULTIPLE_MAPPINGS_FOR_CRITERION_TYPE = 11
        MULTIPLE_MAPPINGS_FOR_PLACEHOLDER_FIELD = 12
        MULTIPLE_MAPPINGS_FOR_CRITERION_FIELD = 13
        UNEXPECTED_ATTRIBUTE_FIELD_MAPPINGS = 14
        LOCATION_PLACEHOLDER_ONLY_FOR_PLACES_FEEDS = 15
        CANNOT_MODIFY_MAPPINGS_FOR_TYPED_FEED = 16
        INVALID_PLACEHOLDER_TYPE_FOR_NON_SYSTEM_GENERATED_FEED = 17
        INVALID_PLACEHOLDER_TYPE_FOR_SYSTEM_GENERATED_FEED_TYPE = 18
        ATTRIBUTE_FIELD_MAPPING_MISSING_FIELD = 19
        LEGACY_FEED_TYPE_READ_ONLY = 20


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import bidding_strategy_type
from google.ads.googleads.v21.enums.types import (
    target_impression_share_location,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "AccessibleBiddingStrategy",
    },
)


class AccessibleBiddingStrategy(proto.Message):
    r"""Represents a view of BiddingStrategies owned by and shared
    with the customer.
    In contrast to BiddingStrategy, this resource includes
    strategies owned by managers of the customer and shared with
    this customer - in addition to strategies owned by this
    customer. This resource does not provide metrics and only
    exposes a limited subset of the BiddingStrategy attributes.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the accessible bidding
            strategy. AccessibleBiddingStrategy resource names have the
            form:

            ``customers/{customer_id}/accessibleBiddingStrategies/{bidding_strategy_id}``
        id (int):
            Output only. The ID of the bidding strategy.
        name (str):
            Output only. The name of the bidding
            strategy.
        type_ (google.ads.googleads.v21.enums.types.BiddingStrategyTypeEnum.BiddingStrategyType):
            Output only. The type of the bidding
            strategy.
        owner_customer_id (int):
            Output only. The ID of the Customer which
            owns the bidding strategy.
        owner_descriptive_name (str):
            Output only. descriptive_name of the Customer which owns the
            bidding strategy.
        maximize_conversion_value (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.MaximizeConversionValue):
            Output only. An automated bidding strategy to
            help get the most conversion value for your
            campaigns while spending your budget.

            This field is a member of `oneof`_ ``scheme``.
        maximize_conversions (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.MaximizeConversions):
            Output only. An automated bidding strategy to
            help get the most conversions for your campaigns
            while spending your budget.

            This field is a member of `oneof`_ ``scheme``.
        target_cpa (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.TargetCpa):
            Output only. A bidding strategy that sets
            bids to help get as many conversions as possible
            at the target cost-per-acquisition (CPA) you
            set.

            This field is a member of `oneof`_ ``scheme``.
        target_impression_share (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.TargetImpressionShare):
            Output only. A bidding strategy that
            automatically optimizes towards a chosen
            percentage of impressions.

            This field is a member of `oneof`_ ``scheme``.
        target_roas (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.TargetRoas):
            Output only. A bidding strategy that helps
            you maximize revenue while averaging a specific
            target Return On Ad Spend (ROAS).

            This field is a member of `oneof`_ ``scheme``.
        target_spend (google.ads.googleads.v21.resources.types.AccessibleBiddingStrategy.TargetSpend):
            Output only. A bid strategy that sets your
            bids to help get as many clicks as possible
            within your budget.

            This field is a member of `oneof`_ ``scheme``.
    """

    class MaximizeConversionValue(proto.Message):
        r"""An automated bidding strategy to help get the most conversion
        value for your campaigns while spending your budget.

        Attributes:
            target_roas (float):
                Output only. The target return on ad spend
                (ROAS) option. If set, the bid strategy will
                maximize revenue while averaging the target
                return on ad spend. If the target ROAS is high,
                the bid strategy may not be able to spend the
                full budget. If the target ROAS is not set, the
                bid strategy will aim to achieve the highest
                possible ROAS for the budget.
        """

        target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
        )

    class MaximizeConversions(proto.Message):
        r"""An automated bidding strategy to help get the most
        conversions for your campaigns while spending your budget.

        Attributes:
            target_cpa_micros (int):
                Output only. The target cost per acquisition
                (CPA) option. This is the average amount that
                you would like to spend per acquisition.
        """

        target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=2,
        )

    class TargetCpa(proto.Message):
        r"""An automated bid strategy that sets bids to help get as many
        conversions as possible at the target cost-per-acquisition (CPA)
        you set.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_cpa_micros (int):
                Output only. Average CPA target.
                This target should be greater than or equal to
                minimum billable unit based on the currency for
                the account.

                This field is a member of `oneof`_ ``_target_cpa_micros``.
        """

        target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )

    class TargetImpressionShare(proto.Message):
        r"""An automated bidding strategy that sets bids so that a
        certain percentage of search ads are shown at the top of the
        first page (or other targeted location).


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            location (google.ads.googleads.v21.enums.types.TargetImpressionShareLocationEnum.TargetImpressionShareLocation):
                Output only. The targeted location on the
                search results page.
            location_fraction_micros (int):
                The chosen fraction of ads to be shown in the
                targeted location in micros. For example, 1%
                equals 10,000.

                This field is a member of `oneof`_ ``_location_fraction_micros``.
            cpc_bid_ceiling_micros (int):
                Output only. The highest CPC bid the
                automated bidding system is permitted to
                specify. This is a required field entered by the
                advertiser that sets the ceiling and specified
                in local micros.

                This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
        """

        location: (
            target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation,
        )
        location_fraction_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )
        cpc_bid_ceiling_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )

    class TargetRoas(proto.Message):
        r"""An automated bidding strategy that helps you maximize revenue
        while averaging a specific target return on ad spend (ROAS).


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_roas (float):
                Output only. The chosen revenue (based on
                conversion data) per unit of spend.

                This field is a member of `oneof`_ ``_target_roas``.
        """

        target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
            optional=True,
        )

    class TargetSpend(proto.Message):
        r"""An automated bid strategy that sets your bids to help get as
        many clicks as possible within your budget.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_spend_micros (int):
                Output only. The spend target under which to
                maximize clicks. A TargetSpend bidder will
                attempt to spend the smaller of this value or
                the natural throttling spend amount.
                If not specified, the budget is used as the
                spend target. This field is deprecated and
                should no longer be used. See
                https://ads-developers.googleblog.com/2020/05/reminder-about-sunset-creation-of.html
                for details.

                This field is a member of `oneof`_ ``_target_spend_micros``.
            cpc_bid_ceiling_micros (int):
                Output only. Maximum bid limit that can be
                set by the bid strategy. The limit applies to
                all keywords managed by the strategy.

                This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
        """

        target_spend_micros: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )
        cpc_bid_ceiling_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    type_: bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType,
        )
    )
    owner_customer_id: int = proto.Field(
        proto.INT64,
        number=5,
    )
    owner_descriptive_name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    maximize_conversion_value: MaximizeConversionValue = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="scheme",
        message=MaximizeConversionValue,
    )
    maximize_conversions: MaximizeConversions = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="scheme",
        message=MaximizeConversions,
    )
    target_cpa: TargetCpa = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="scheme",
        message=TargetCpa,
    )
    target_impression_share: TargetImpressionShare = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="scheme",
        message=TargetImpressionShare,
    )
    target_roas: TargetRoas = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="scheme",
        message=TargetRoas,
    )
    target_spend: TargetSpend = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="scheme",
        message=TargetSpend,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A gRPC Interceptor that is responsible for handling Google Ads API errors.

This class is initialized in the GoogleAdsClient and passed into a grpc
intercept_channel whenever a new service is initialized. It intercepts requests
to determine if a non-retryable Google Ads API error has been encountered. If
so it translates the error to a GoogleAdsFailure instance and raises it.
"""

import grpc

from grpc import UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor

from .interceptor import Interceptor
from .response_wrappers import _UnaryStreamWrapper, _UnaryUnaryWrapper


class ExceptionInterceptor(
    Interceptor, UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor
):
    """An interceptor that wraps rpc exceptions."""

    def __init__(self, api_version, use_proto_plus=False):
        """Initializes the ExceptionInterceptor.

        Args:
            api_version: a str of the API version of the request.
            use_proto_plus: a boolean of whether returned messages should be
                proto_plus or protobuf.
        """
        super().__init__(api_version)
        self._api_version = api_version
        self._use_proto_plus = use_proto_plus

    def _handle_grpc_failure(self, response):
        """Attempts to convert failed responses to a GoogleAdsException object.

        Handles failed gRPC responses of by attempting to convert them
        to a more readable GoogleAdsException. Certain types of exceptions are
        not converted; if the object's trailing metadata does not indicate that
        it is a GoogleAdsException, or if it falls under a certain category of
        status code, (INTERNAL or RESOURCE_EXHAUSTED). See documentation for
        more information about gRPC status codes:
        https://github.com/grpc/grpc/blob/master/doc/statuscodes.md

        Args:
            response: a grpc.Call/grpc.Future instance.

        Raises:
            GoogleAdsException: If the exception's trailing metadata
                indicates that it is a GoogleAdsException.
            RpcError: If the exception's is a gRPC exception but the trailing
                metadata is empty or is not indicative of a GoogleAdsException,
                or if the exception has a status code of INTERNAL or
                RESOURCE_EXHAUSTED.
            Exception: If not a GoogleAdsException or RpcException the error
                will be raised as-is.
        """
        raise self._get_error_from_response(response)

    def intercept_unary_unary(self, continuation, client_call_details, request):
        """Intercepts and wraps exceptions in the rpc response.

        Overrides abstract method defined in grpc.UnaryUnaryClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call instance representing a service response.

        Raises:
            GoogleAdsException: If the exception's trailing metadata
                indicates that it is a GoogleAdsException.
            RpcError: If the exception's trailing metadata is empty or is not
                indicative of a GoogleAdsException, or if the exception has a
                status code of INTERNAL or RESOURCE_EXHAUSTED.
        """
        response = continuation(client_call_details, request)
        exception = response.exception()

        if exception:
            self._handle_grpc_failure(response)
        else:
            return _UnaryUnaryWrapper(
                response, use_proto_plus=self._use_proto_plus
            )

    def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        """Intercepts and wraps exceptions in the rpc response.

        Overrides abstract method defined in grpc.UnaryStreamClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call instance representing a service response.

        Raises:
            GoogleAdsException: If the exception's trailing metadata
                indicates that it is a GoogleAdsException.
            RpcError: If the exception's trailing metadata is empty or is not
                indicative of a GoogleAdsException, or if the exception has a
                status code of INTERNAL or RESOURCE_EXHAUSTED.
        """
        response = continuation(client_call_details, request)
        return _UnaryStreamWrapper(
            response,
            self._handle_grpc_failure,
            use_proto_plus=self._use_proto_plus,
        )
